package com.github.gkutiel.filter;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.github.gkutiel.filter.Injector.MultipartInjector;
import com.github.gkutiel.filter.Injector.UrlEncodedInjector;

class Engine {

	private final HttpServletRequest req;
	private final HttpServletResponse res;

	public Engine(final HttpServletRequest req, final HttpServletResponse res) {
		this.req = req;
		this.res = res;
	}

	public void invoke(final Class<? extends Filter> filterClass) {
		invoke(getInjector(), null, filterChain(filterClass));
	}

	private List<Class<? extends Filter>> filterChain(final Class<?> c) {
		if (c == null) return new ArrayList<>();
		final List<Class<? extends Filter>> classChain = filterChain(c.getEnclosingClass());
		if (Filter.class.isAssignableFrom(c)) classChain.add(c.asSubclass(Filter.class));
		return classChain;
	}

	private Filter getFilter(final Filter outter, final Class<? extends Filter> filterClass) {
		try {
			if (outter == null) return filterClass.newInstance();
			return filterClass.getConstructor(outter.getClass()).newInstance(outter);
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	private Injector getInjector() {
		if (req.getContentType().startsWith("multipart/form-data;")) return new MultipartInjector(req, res);
		return new UrlEncodedInjector(req, res);
	}

	private void invoke(final Injector injector, final Filter outter, final List<Class<? extends Filter>> chain) {
		if (outter != null) outter.apply();
		if (chain.isEmpty()) return;
		final Class<? extends Filter> filterClass = chain.remove(0);
		invoke(injector, newFilter(injector, outter, filterClass), chain);
	}

	private Filter newFilter(final Injector injector, final Filter outter, final Class<? extends Filter> filterClass) {
		try {
			return injector.inject(getFilter(outter, filterClass));
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}
}
